@@ -7,6 +7,7 @@ from shutil import copyfile
 import numpy as np
 from Bio import AlignIO, SeqIO, Seq, Align
 from .utils import run_shell_command, nthreads_value, shquote
+from collections import defaultdict
 
 class AlignmentError(Exception):
     # TODO: this exception should potentially be renamed and made augur-wide
@@ -69,7 +70,8 @@ def run(args):
         if args.reference_sequence and not existing_aln:
             seqs_to_align_fname = args.output+".to_align.fasta"
             ref_seq = read_reference(args.reference_sequence)
-            write_seqs(list(seqs.values())+[ref_seq], seqs_to_align_fname)
+            # reference sequence needs to be the first one for auto direction adjustment (auto reverse-complement)
+            write_seqs([ref_seq] + list(seqs.values()), seqs_to_align_fname)
             ref_name = ref_seq.id
         elif existing_aln:
             seqs_to_align_fname = args.output+".new_seqs_to_align.fasta"
@@ -96,17 +98,24 @@ def run(args):
         if args.debug:
             copyfile(args.output, args.output+".post_aligner.fasta")
 
-        # convert the aligner output to upper case (replacing the file in place)
-        write_uppercase_alignment_in_place(args.output)
+        # reads the new alignment
+        seqs = read_alignment(args.output)
+
+        # convert the aligner output to upper case and remove auto reverse-complement prefix
+        prettify_alignment(seqs)
 
         # if we've specified a reference, strip out all the columns not present in the reference
         # this will overwrite the alignment file
         if ref_name:
-            seqs = strip_non_reference(args.output, ref_name, keep_reference=not args.remove_reference)
+            seqs = strip_non_reference(seqs, ref_name, insertion_csv=args.output+".insertions.csv")
+            if args.remove_reference:
+                seqs = remove_reference_sequence(seqs, ref_name)
             write_seqs(seqs, args.output)
         if args.fill_gaps:
             make_gaps_ambiguous(seqs)
-            write_seqs(seqs, args.output)
+
+        # write the modified sequences back to the alignment file
+        write_seqs(seqs, args.output)
 
 
     except AlignmentError as e:
@@ -124,13 +133,12 @@ def read_sequences(*fnames):
     try:
         for fname in fnames:
             for record in SeqIO.parse(fname, 'fasta'):
-                if record.name in seqs:
-                    if str(record) != str(seqs[record.name]):
-                        raise AlignmentError("Detected duplicate input strains \"%s\" but the sequences are different."%record.name)
+                if record.name in seqs and record.seq != seqs[record.name].seq:
+                    raise AlignmentError("Detected duplicate input strains \"%s\" but the sequences are different." % record.name)
                     # if the same sequence then we can proceed (and we only take one)
                 seqs[record.name] = record
     except FileNotFoundError:
-        raise AlignmentError("\nCannot read sequences -- make sure the file %s exists and contains sequences in fasta format"%fname)
+        raise AlignmentError("\nCannot read sequences -- make sure the file %s exists and contains sequences in fasta format" % fname)
     except ValueError as error:
         raise AlignmentError("\nERROR: Problem reading in {}: {}".format(fname, str(error)))
     return seqs
@@ -162,6 +170,7 @@ def ensure_reference_strain_present(ref_name, existing_alignment, seqs):
     #     shoutput = shquote(output)
     #     shname = shquote(seq_fname)
     #     cmd = "mafft --reorder --anysymbol --thread %d %s 1> %s 2> %s.log"%(args.nthreads, shname, shoutput, shoutput)
+
 def read_reference(ref_fname):
     if not os.path.isfile(ref_fname):
         raise AlignmentError("ERROR: Cannot read reference sequence."
@@ -176,9 +185,9 @@ def read_reference(ref_fname):
 def generate_alignment_cmd(method, nthreads, existing_aln_fname, seqs_to_align_fname, aln_fname, log_fname):
     if method=='mafft':
         if existing_aln_fname:
-            cmd = "mafft --add %s --keeplength --reorder --anysymbol --nomemsave --thread %d %s 1> %s 2> %s"%(shquote(seqs_to_align_fname), nthreads, shquote(existing_aln_fname), shquote(aln_fname), shquote(log_fname))
+            cmd = "mafft --add %s --keeplength --reorder --anysymbol --nomemsave --adjustdirection --thread %d %s 1> %s 2> %s"%(shquote(seqs_to_align_fname), nthreads, shquote(existing_aln_fname), shquote(aln_fname), shquote(log_fname))
         else:
-            cmd = "mafft --reorder --anysymbol --nomemsave --thread %d %s 1> %s 2> %s"%(nthreads, shquote(seqs_to_align_fname), shquote(aln_fname), shquote(log_fname))
+            cmd = "mafft --reorder --anysymbol --nomemsave --adjustdirection --thread %d %s 1> %s 2> %s"%(nthreads, shquote(seqs_to_align_fname), shquote(aln_fname), shquote(log_fname))
         print("\nusing mafft to align via:\n\t" + cmd +
               " \n\n\tKatoh et al, Nucleic Acid Research, vol 30, issue 14"
               "\n\thttps://doi.org/10.1093%2Fnar%2Fgkf436\n")
@@ -187,54 +196,131 @@ def generate_alignment_cmd(method, nthreads, existing_aln_fname, seqs_to_align_f
     return cmd
 
 
-def write_uppercase_alignment_in_place(fname):
-    aln = AlignIO.read(fname, 'fasta')
-    for seq in aln:
-        seq.seq = seq.seq.upper()
-    AlignIO.write(aln, fname, 'fasta')
+def remove_reference_sequence(seqs, reference_name):
+    return [seq for seq in seqs if seq.name!=reference_name]
 
 
-def strip_non_reference(alignment_fname, reference, keep_reference=False):
+def strip_non_reference(aln, reference, insertion_csv=None):
     '''
     return sequences that have all insertions relative to the reference
-    removed. The alignment is read from file and returned as list of sequences.
+    removed. The aligment is returned as list of sequences.
 
     Parameters
     ----------
-    alignment_fname : str
-        alignment file name, file needs to be fasta format
+    aln : MultipleSeqAlign
+        Biopython Alignment
     reference : str
         name of reference sequence, assumed to be part of the alignment
-    keep_reference : bool, optional
-        by default, the reference sequence is removed after stripping
-        non-reference sequence. To keep the reference, use keep_reference=True
 
     Returns
     -------
     list
         list of trimmed sequences, effectively a multiple alignment
+
+    Tests
+    -----
+    >>> [s.name for s in strip_non_reference(read_alignment("tests/data/align/test_aligned_sequences.fasta"), "with_gaps")]
+    Trimmed gaps in with_gaps from the alignment
+    ['with_gaps', 'no_gaps', 'some_other_seq', '_R_crick_strand']
+    >>> [s.name for s in strip_non_reference(read_alignment("tests/data/align/test_aligned_sequences.fasta"), "no_gaps")]
+    No gaps in alignment to trim (with respect to the reference, no_gaps)
+    ['with_gaps', 'no_gaps', 'some_other_seq', '_R_crick_strand']
+    >>> [s.name for s in strip_non_reference(read_alignment("tests/data/align/test_aligned_sequences.fasta"), "missing")]
+    Traceback (most recent call last):
+      ...
+    augur.align.AlignmentError: ERROR: reference missing not found in alignment
     '''
-    aln = AlignIO.read(alignment_fname, 'fasta')
     seqs = {s.name:s for s in aln}
     if reference in seqs:
         ref_array = np.array(seqs[reference])
         if "-" not in ref_array:
             print("No gaps in alignment to trim (with respect to the reference, %s)"%reference)
-            return aln
         ungapped = ref_array!='-'
         ref_aln_array = np.array(aln)[:,ungapped]
     else:
         raise AlignmentError("ERROR: reference %s not found in alignment"%reference)
 
+    if False in ungapped and insertion_csv:
+        analyse_insertions(aln, ungapped, insertion_csv)
+
     out_seqs = []
     for seq, seq_array in zip(aln, ref_aln_array):
         seq.seq = Seq.Seq(''.join(seq_array))
-        if keep_reference or seq.name!=reference:
-            out_seqs.append(seq)
+        out_seqs.append(seq)
+
+    if "-" in ref_array:
+        print("Trimmed gaps in", reference, "from the alignment")
 
-    print("Trimmed gaps in", reference, "from the alignment")
     return out_seqs
 
+def analyse_insertions(aln, ungapped, insertion_csv):
+    ## Gather groups (runs) of insertions:
+    insertion_coords = [] # python syntax - e.g. [0, 3, 5] means indexes 0,1 & 2 are insertions (w.r.t. ref), to the right of 0-based ref pos 5
+    _open_idx = False
+    _ref_idx = -1
+    for i, in_ref in enumerate(ungapped):
+        if not in_ref and _open_idx is False:
+            _open_idx = i # insertion run start
+        elif in_ref and _open_idx is not False:
+            insertion_coords.append([_open_idx, i, _ref_idx])# insertion run has finished
+            _open_idx = False
+        if in_ref:
+            _ref_idx += 1
+    if _open_idx is not False:
+        insertion_coords.append([_open_idx, len(ungapped), _ref_idx])
+
+    # For each run of insertions (w.r.t. reference) collect the insertions we have
+    insertions = [defaultdict(list) for ins in insertion_coords]
+    for idx, insertion_coord in enumerate(insertion_coords):
+        for seq in aln:
+            s = seq[insertion_coord[0]:insertion_coord[1]].seq.ungap("-").ungap("N").ungap("?")
+            if len(s):
+                insertions[idx][str(s)].append(seq.name)
+
+    for insertion_coord, data in zip(insertion_coords, insertions):
+        # GFF is 1-based & insertions are to the right of the base.
+        print("{}bp insertion at ref position {}".format(insertion_coord[1]-insertion_coord[0], insertion_coord[2]+1))
+        for k, v in data.items():
+            print("\t{}: {}".format(k, ", ".join(v)))
+        if not len(data.keys()):
+            # This happens when there _is_ an insertion, but it's an insertion of gaps
+            # We know that these are associated with poor alignments...
+            print("\tWARNING: this insertion was caused due to 'N's or '?'s in provided sequences")
+
+    # output for auspice drag&drop -- GFF is 1-based & insertions are to the right of the base.
+    header = ["strain"]+["insertion: {}bp @ ref pos {}".format(ic[1]-ic[0], ic[2]+1) for ic in insertion_coords]
+    strain_data = defaultdict(lambda: ["" for _ in range(0, len(insertion_coords))])
+    for idx, i_data in enumerate(insertions):
+        for insertion_seq, strains in i_data.items():
+            for strain in strains:
+                strain_data[strain][idx] = insertion_seq
+    with open(insertion_csv, 'w', encoding='utf-8') as fh:
+        print(",".join(header), file=fh)
+        for strain in strain_data:
+            print("{},{}".format(strain, ",".join(strain_data[strain])), file=fh)
+
+
+def prettify_alignment(aln):
+    '''
+    Converts all bases to uppercase and removes auto reverse-complement prefix (_R_).
+    This modifies the alignment in place.
+
+    Parameters
+    ----------
+    aln : MultipleSeqAlign
+        Biopython Alignment
+    '''
+    for seq in aln:
+        seq.seq = seq.seq.upper()
+        # AlignIO.read repeats the ID in the name and description field
+        if seq.id.startswith("_R_"):
+            seq.id = seq.id[3:]
+            print("Sequence \"{}\" was reverse-complemented by the alignment program.".format(seq.id))
+        if seq.name.startswith("_R_"):
+            seq.name = seq.name[3:]
+        if seq.description.startswith("_R_"):
+            seq.description = seq.description[3:]
+
 
 def make_gaps_ambiguous(aln):
     '''
@@ -248,10 +334,10 @@ def make_gaps_ambiguous(aln):
         Biopython Alignment
     '''
     for seq in aln:
-        seq_array = np.array(seq)
-        gaps = seq_array=='-'
-        seq_array[gaps]='N'
-        seq.seq = Seq.Seq("".join(seq_array))
+        _seq = str(seq.seq)
+        _seq = _seq.replace('-', 'N')
+        seq.seq = Seq.Seq(_seq, alphabet=seq.seq.alphabet)
+
 
 def check_duplicates(*values):
     names = set()
